# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

import pytest

from ....testing import example_data
from ...niftyreg import get_custom_path
from ...niftyreg.tests.test_regutils import no_nifty_tool
from ..qt1 import FitQt1


@pytest.mark.skipif(no_nifty_tool(cmd="fit_qt1"), reason="niftyfit is not installed")
def test_fit_qt1():
    """ Testing FitQt1 interface."""
    # Create a node object
    fit_qt1 = FitQt1()

    # Check if the command is properly defined
    cmd = get_custom_path("fit_qt1", env_dir="NIFTYFITDIR")
    assert fit_qt1.cmd == cmd

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        fit_qt1.run()

    # Regular test:
    in_file = example_data("TI4D.nii.gz")
    fit_qt1.inputs.source_file = in_file

    cmd_tmp = "{cmd} -source {in_file} -comp {comp} -error {error} -m0map \
{map0} -mcmap {cmap} -res {res} -syn {syn} -t1map {t1map}"

    expected_cmd = cmd_tmp.format(
        cmd=cmd,
        in_file=in_file,
        comp="TI4D_comp.nii.gz",
        map0="TI4D_m0map.nii.gz",
        error="TI4D_error.nii.gz",
        cmap="TI4D_mcmap.nii.gz",
        res="TI4D_res.nii.gz",
        t1map="TI4D_t1map.nii.gz",
        syn="TI4D_syn.nii.gz",
    )

    assert fit_qt1.cmdline == expected_cmd

    # Runs T1 fitting to inversion and saturation recovery data (NLSQR)
    fit_qt1_2 = FitQt1(tis=[1, 2, 5], ir_flag=True)
    in_file = example_data("TI4D.nii.gz")
    fit_qt1_2.inputs.source_file = in_file

    cmd_tmp = "{cmd} -source {in_file} -IR -TIs 1.0 2.0 5.0 \
-comp {comp} -error {error} -m0map {map0} -mcmap {cmap} -res {res} \
-syn {syn} -t1map {t1map}"

    expected_cmd = cmd_tmp.format(
        cmd=cmd,
        in_file=in_file,
        comp="TI4D_comp.nii.gz",
        map0="TI4D_m0map.nii.gz",
        error="TI4D_error.nii.gz",
        cmap="TI4D_mcmap.nii.gz",
        res="TI4D_res.nii.gz",
        t1map="TI4D_t1map.nii.gz",
        syn="TI4D_syn.nii.gz",
    )

    assert fit_qt1_2.cmdline == expected_cmd

    # Runs T1 fitting to spoiled gradient echo (SPGR) data (NLSQR)
    fit_qt1_3 = FitQt1(flips=[2, 4, 8], spgr=True)
    in_file = example_data("TI4D.nii.gz")
    fit_qt1_3.inputs.source_file = in_file

    cmd_tmp = "{cmd} -source {in_file} -comp {comp} -error {error} \
-flips 2.0 4.0 8.0 -m0map {map0} -mcmap {cmap} -res {res} -SPGR -syn {syn} \
-t1map {t1map}"

    expected_cmd = cmd_tmp.format(
        cmd=cmd,
        in_file=in_file,
        comp="TI4D_comp.nii.gz",
        map0="TI4D_m0map.nii.gz",
        error="TI4D_error.nii.gz",
        cmap="TI4D_mcmap.nii.gz",
        res="TI4D_res.nii.gz",
        t1map="TI4D_t1map.nii.gz",
        syn="TI4D_syn.nii.gz",
    )

    assert fit_qt1_3.cmdline == expected_cmd



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import Resample


def test_Resample_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="-inset %s",
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        master=dict(
            argstr="-master %s",
            extensions=None,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        orientation=dict(
            argstr="-orient %s",
        ),
        out_file=dict(
            argstr="-prefix %s",
            extensions=None,
            name_source="in_file",
            name_template="%s_resample",
        ),
        outputtype=dict(),
        resample_mode=dict(
            argstr="-rmode %s",
        ),
        voxel_size=dict(
            argstr="-dxyz %f %f %f",
        ),
    )
    inputs = Resample.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Resample_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = Resample.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

import os

from ...base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)


class BRAINSPosteriorToContinuousClassInputSpec(CommandLineInputSpec):
    inputWhiteVolume = File(
        desc="White Matter Posterior Volume",
        exists=True,
        argstr="--inputWhiteVolume %s",
    )
    inputBasalGmVolume = File(
        desc="Basal Grey Matter Posterior Volume",
        exists=True,
        argstr="--inputBasalGmVolume %s",
    )
    inputSurfaceGmVolume = File(
        desc="Surface Grey Matter Posterior Volume",
        exists=True,
        argstr="--inputSurfaceGmVolume %s",
    )
    inputCsfVolume = File(
        desc="CSF Posterior Volume", exists=True, argstr="--inputCsfVolume %s"
    )
    inputVbVolume = File(
        desc="Venous Blood Posterior Volume", exists=True, argstr="--inputVbVolume %s"
    )
    inputCrblGmVolume = File(
        desc="Cerebellum Grey Matter Posterior Volume",
        exists=True,
        argstr="--inputCrblGmVolume %s",
    )
    inputCrblWmVolume = File(
        desc="Cerebellum White Matter Posterior Volume",
        exists=True,
        argstr="--inputCrblWmVolume %s",
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Output Continuous Tissue Classified Image",
        argstr="--outputVolume %s",
    )


class BRAINSPosteriorToContinuousClassOutputSpec(TraitedSpec):
    outputVolume = File(desc="Output Continuous Tissue Classified Image", exists=True)


class BRAINSPosteriorToContinuousClass(SEMLikeCommandLine):
    """title: Tissue Classification

category: BRAINS.Classify

description: This program will generate an 8-bit continuous tissue classified image based on BRAINSABC posterior images.

version: 3.0

documentation-url: http://www.nitrc.org/plugins/mwiki/index.php/brains:BRAINSClassify

license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

contributor: Vincent A. Magnotta

acknowledgements: Funding for this work was provided by NIH/NINDS award NS050568

"""

    input_spec = BRAINSPosteriorToContinuousClassInputSpec
    output_spec = BRAINSPosteriorToContinuousClassOutputSpec
    _cmd = " BRAINSPosteriorToContinuousClass "
    _outputs_filenames = {"outputVolume": "outputVolume"}
    _redirect_x = False



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import Edge3


def test_Edge3_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        datum=dict(
            argstr="-datum %s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fscale=dict(
            argstr="-fscale",
            xor=["gscale", "nscale", "scale_floats"],
        ),
        gscale=dict(
            argstr="-gscale",
            xor=["fscale", "nscale", "scale_floats"],
        ),
        in_file=dict(
            argstr="-input %s",
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=0,
        ),
        nscale=dict(
            argstr="-nscale",
            xor=["fscale", "gscale", "scale_floats"],
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_file=dict(
            argstr="-prefix %s",
            extensions=None,
            position=-1,
        ),
        outputtype=dict(),
        scale_floats=dict(
            argstr="-scale_floats %f",
            xor=["fscale", "gscale", "nscale"],
        ),
        verbose=dict(
            argstr="-verbose",
        ),
    )
    inputs = Edge3.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Edge3_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = Edge3.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..minc import Beast


def test_Beast_inputs():
    input_map = dict(
        abspath=dict(
            argstr="-abspath",
            usedefault=True,
        ),
        args=dict(
            argstr="%s",
        ),
        clobber=dict(
            argstr="-clobber",
            usedefault=True,
        ),
        confidence_level_alpha=dict(
            argstr="-alpha %s",
            usedefault=True,
        ),
        configuration_file=dict(
            argstr="-configuration %s",
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fill_holes=dict(
            argstr="-fill",
        ),
        flip_images=dict(
            argstr="-flip",
        ),
        input_file=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        library_dir=dict(
            argstr="%s",
            mandatory=True,
            position=-3,
        ),
        load_moments=dict(
            argstr="-load_moments",
        ),
        median_filter=dict(
            argstr="-median",
        ),
        nlm_filter=dict(
            argstr="-nlm_filter",
        ),
        number_selected_images=dict(
            argstr="-selection_num %s",
            usedefault=True,
        ),
        output_file=dict(
            argstr="%s",
            extensions=None,
            hash_files=False,
            name_source=["input_file"],
            name_template="%s_beast_mask.mnc",
            position=-1,
        ),
        patch_size=dict(
            argstr="-patch_size %s",
            usedefault=True,
        ),
        probability_map=dict(
            argstr="-probability",
        ),
        same_resolution=dict(
            argstr="-same_resolution",
        ),
        search_area=dict(
            argstr="-search_area %s",
            usedefault=True,
        ),
        smoothness_factor_beta=dict(
            argstr="-beta %s",
            usedefault=True,
        ),
        threshold_patch_selection=dict(
            argstr="-threshold %s",
            usedefault=True,
        ),
        voxel_size=dict(
            argstr="-voxel_size %s",
            usedefault=True,
        ),
    )
    inputs = Beast.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Beast_outputs():
    output_map = dict(
        output_file=dict(
            extensions=None,
        ),
    )
    outputs = Beast.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..reg import RegF3D


def test_RegF3D_inputs():
    input_map = dict(
        aff_file=dict(
            argstr="-aff %s",
            extensions=None,
        ),
        amc_flag=dict(
            argstr="-amc",
        ),
        args=dict(
            argstr="%s",
        ),
        be_val=dict(
            argstr="-be %f",
        ),
        cpp_file=dict(
            argstr="-cpp %s",
            extensions=None,
            name_source=["flo_file"],
            name_template="%s_cpp.nii.gz",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fbn2_val=dict(
            argstr="-fbn %d %d",
        ),
        fbn_val=dict(
            argstr="--fbn %d",
        ),
        flo_file=dict(
            argstr="-flo %s",
            extensions=None,
            mandatory=True,
        ),
        flo_smooth_val=dict(
            argstr="-smooF %f",
        ),
        flwth2_thr_val=dict(
            argstr="-fLwTh %d %f",
        ),
        flwth_thr_val=dict(
            argstr="--fLwTh %f",
        ),
        fmask_file=dict(
            argstr="-fmask %s",
            extensions=None,
        ),
        fupth2_thr_val=dict(
            argstr="-fUpTh %d %f",
        ),
        fupth_thr_val=dict(
            argstr="--fUpTh %f",
        ),
        incpp_file=dict(
            argstr="-incpp %s",
            extensions=None,
        ),
        jl_val=dict(
            argstr="-jl %f",
        ),
        kld2_flag=dict(
            argstr="-kld %d",
        ),
        kld_flag=dict(
            argstr="--kld",
        ),
        le_val=dict(
            argstr="-le %f",
        ),
        ln_val=dict(
            argstr="-ln %d",
        ),
        lncc2_val=dict(
            argstr="-lncc %d %f",
        ),
        lncc_val=dict(
            argstr="--lncc %f",
        ),
        lp_val=dict(
            argstr="-lp %d",
        ),
        maxit_val=dict(
            argstr="-maxit %d",
        ),
        nmi_flag=dict(
            argstr="--nmi",
        ),
        no_app_jl_flag=dict(
            argstr="-noAppJL",
        ),
        noconj_flag=dict(
            argstr="-noConj",
        ),
        nopy_flag=dict(
            argstr="-nopy",
        ),
        nox_flag=dict(
            argstr="-nox",
        ),
        noy_flag=dict(
            argstr="-noy",
        ),
        noz_flag=dict(
            argstr="-noz",
        ),
        omp_core_val=dict(
            argstr="-omp %i",
            usedefault=True,
        ),
        pad_val=dict(
            argstr="-pad %f",
        ),
        pert_val=dict(
            argstr="-pert %d",
        ),
        rbn2_val=dict(
            argstr="-rbn %d %d",
        ),
        rbn_val=dict(
            argstr="--rbn %d",
        ),
        ref_file=dict(
            argstr="-ref %s",
            extensions=None,
            mandatory=True,
        ),
        ref_smooth_val=dict(
            argstr="-smooR %f",
        ),
        res_file=dict(
            argstr="-res %s",
            extensions=None,
            name_source=["flo_file"],
            name_template="%s_res.nii.gz",
        ),
        rlwth2_thr_val=dict(
            argstr="-rLwTh %d %f",
        ),
        rlwth_thr_val=dict(
            argstr="--rLwTh %f",
        ),
        rmask_file=dict(
            argstr="-rmask %s",
            extensions=None,
        ),
        rupth2_thr_val=dict(
            argstr="-rUpTh %d %f",
        ),
        rupth_thr_val=dict(
            argstr="--rUpTh %f",
        ),
        smooth_grad_val=dict(
            argstr="-smoothGrad %f",
        ),
        ssd2_flag=dict(
            argstr="-ssd %d",
        ),
        ssd_flag=dict(
            argstr="--ssd",
        ),
        sx_val=dict(
            argstr="-sx %f",
        ),
        sy_val=dict(
            argstr="-sy %f",
        ),
        sz_val=dict(
            argstr="-sz %f",
        ),
        vel_flag=dict(
            argstr="-vel",
        ),
        verbosity_off_flag=dict(
            argstr="-voff",
        ),
    )
    inputs = RegF3D.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_RegF3D_outputs():
    output_map = dict(
        avg_output=dict(),
        cpp_file=dict(
            extensions=None,
        ),
        invcpp_file=dict(
            extensions=None,
        ),
        invres_file=dict(
            extensions=None,
        ),
        res_file=dict(
            extensions=None,
        ),
    )
    outputs = RegF3D.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..model import Binarize


def test_Binarize_inputs():
    input_map = dict(
        abs=dict(
            argstr="--abs",
        ),
        args=dict(
            argstr="%s",
        ),
        bin_col_num=dict(
            argstr="--bincol",
        ),
        bin_val=dict(
            argstr="--binval %d",
        ),
        bin_val_not=dict(
            argstr="--binvalnot %d",
        ),
        binary_file=dict(
            argstr="--o %s",
            extensions=None,
            genfile=True,
        ),
        count_file=dict(
            argstr="--count %s",
        ),
        dilate=dict(
            argstr="--dilate %d",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        erode=dict(
            argstr="--erode  %d",
        ),
        erode2d=dict(
            argstr="--erode2d %d",
        ),
        frame_no=dict(
            argstr="--frame %s",
        ),
        in_file=dict(
            argstr="--i %s",
            copyfile=False,
            extensions=None,
            mandatory=True,
        ),
        invert=dict(
            argstr="--inv",
        ),
        mask_file=dict(
            argstr="--mask maskvol",
            extensions=None,
        ),
        mask_thresh=dict(
            argstr="--mask-thresh %f",
        ),
        match=dict(
            argstr="--match %d...",
        ),
        max=dict(
            argstr="--max %f",
            xor=["wm_ven_csf"],
        ),
        merge_file=dict(
            argstr="--merge %s",
            extensions=None,
        ),
        min=dict(
            argstr="--min %f",
            xor=["wm_ven_csf"],
        ),
        out_type=dict(
            argstr="",
        ),
        rmax=dict(
            argstr="--rmax %f",
        ),
        rmin=dict(
            argstr="--rmin %f",
        ),
        subjects_dir=dict(),
        ventricles=dict(
            argstr="--ventricles",
        ),
        wm=dict(
            argstr="--wm",
        ),
        wm_ven_csf=dict(
            argstr="--wm+vcsf",
            xor=["min", "max"],
        ),
        zero_edges=dict(
            argstr="--zero-edges",
        ),
        zero_slice_edge=dict(
            argstr="--zero-slice-edges",
        ),
    )
    inputs = Binarize.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Binarize_outputs():
    output_map = dict(
        binary_file=dict(
            extensions=None,
        ),
        count_file=dict(
            extensions=None,
        ),
    )
    outputs = Binarize.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..preprocess import TSmooth


def test_TSmooth_inputs():
    input_map = dict(
        adaptive=dict(
            argstr="-adaptive %d",
        ),
        args=dict(
            argstr="%s",
        ),
        blackman=dict(
            argstr="-blackman %d",
        ),
        custom=dict(
            argstr="-custom %s",
            extensions=None,
        ),
        datum=dict(
            argstr="-datum %s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        hamming=dict(
            argstr="-hamming %d",
        ),
        in_file=dict(
            argstr="%s",
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        lin=dict(
            argstr="-lin",
        ),
        lin3=dict(
            argstr="-3lin %d",
        ),
        med=dict(
            argstr="-med",
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        osf=dict(
            argstr="-osf",
        ),
        out_file=dict(
            argstr="-prefix %s",
            extensions=None,
            name_source="in_file",
            name_template="%s_smooth",
        ),
        outputtype=dict(),
    )
    inputs = TSmooth.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_TSmooth_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = TSmooth.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..preprocess import OutlierCount


def test_OutlierCount_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        autoclip=dict(
            argstr="-autoclip",
            usedefault=True,
            xor=["mask"],
        ),
        automask=dict(
            argstr="-automask",
            usedefault=True,
            xor=["mask"],
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fraction=dict(
            argstr="-fraction",
            usedefault=True,
        ),
        in_file=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        interval=dict(
            argstr="-range",
            usedefault=True,
        ),
        legendre=dict(
            argstr="-legendre",
            usedefault=True,
        ),
        mask=dict(
            argstr="-mask %s",
            extensions=None,
            xor=["autoclip", "automask"],
        ),
        out_file=dict(
            extensions=None,
            keep_extension=False,
            name_source=["in_file"],
            name_template="%s_outliers",
        ),
        outliers_file=dict(
            argstr="-save %s",
            extensions=None,
            keep_extension=True,
            name_source=["in_file"],
            name_template="%s_outliers",
            output_name="out_outliers",
        ),
        polort=dict(
            argstr="-polort %d",
        ),
        qthr=dict(
            argstr="-qthr %.5f",
            usedefault=True,
        ),
        save_outliers=dict(
            usedefault=True,
        ),
    )
    inputs = OutlierCount.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_OutlierCount_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
        out_outliers=dict(
            extensions=None,
        ),
    )
    outputs = OutlierCount.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..misc import SplitROIs


def test_SplitROIs_inputs():
    input_map = dict(
        in_file=dict(
            extensions=None,
            mandatory=True,
        ),
        in_mask=dict(
            extensions=None,
        ),
        roi_size=dict(),
    )
    inputs = SplitROIs.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_SplitROIs_outputs():
    output_map = dict(
        out_files=dict(),
        out_index=dict(),
        out_masks=dict(),
    )
    outputs = SplitROIs.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import FilterRegressor


def test_FilterRegressor_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        design_file=dict(
            argstr="-d %s",
            extensions=None,
            mandatory=True,
            position=3,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        filter_all=dict(
            argstr="-f '%s'",
            mandatory=True,
            position=4,
            xor=["filter_columns"],
        ),
        filter_columns=dict(
            argstr="-f '%s'",
            mandatory=True,
            position=4,
            xor=["filter_all"],
        ),
        in_file=dict(
            argstr="-i %s",
            extensions=None,
            mandatory=True,
            position=1,
        ),
        mask=dict(
            argstr="-m %s",
            extensions=None,
        ),
        out_file=dict(
            argstr="-o %s",
            extensions=None,
            genfile=True,
            hash_files=False,
            position=2,
        ),
        out_vnscales=dict(
            argstr="--out_vnscales",
        ),
        output_type=dict(),
        var_norm=dict(
            argstr="--vn",
        ),
    )
    inputs = FilterRegressor.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_FilterRegressor_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = FilterRegressor.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..convert import DT2NIfTI


def test_DT2NIfTI_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        header_file=dict(
            argstr="-header %s",
            extensions=None,
            mandatory=True,
            position=3,
        ),
        in_file=dict(
            argstr="-inputfile %s",
            extensions=None,
            mandatory=True,
            position=1,
        ),
        output_root=dict(
            argstr="-outputroot %s",
            extensions=None,
            genfile=True,
            position=2,
        ),
    )
    inputs = DT2NIfTI.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_DT2NIfTI_outputs():
    output_map = dict(
        dt=dict(
            extensions=None,
        ),
        exitcode=dict(
            extensions=None,
        ),
        lns0=dict(
            extensions=None,
        ),
    )
    outputs = DT2NIfTI.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)
import os


class VotingBinaryHoleFillingImageFilterInputSpec(CommandLineInputSpec):
    radius = InputMultiPath(
        traits.Int,
        desc="The radius of a hole to be filled",
        sep=",",
        argstr="--radius %s",
    )
    majorityThreshold = traits.Int(
        desc="The number of pixels over 50% that will decide whether an OFF pixel will become ON or not. For example, if the neighborhood of a pixel has 124 pixels (excluding itself), the 50% will be 62, and if you set a Majority threshold of 5, that means that the filter will require 67 or more neighbor pixels to be ON in order to switch the current OFF pixel to ON.",
        argstr="--majorityThreshold %d",
    )
    background = traits.Int(
        desc="The value associated with the background (not object)",
        argstr="--background %d",
    )
    foreground = traits.Int(
        desc="The value associated with the foreground (object)",
        argstr="--foreground %d",
    )
    inputVolume = File(
        position=-2, desc="Input volume to be filtered", exists=True, argstr="%s"
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Output filtered",
        argstr="%s",
    )


class VotingBinaryHoleFillingImageFilterOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Output filtered", exists=True)


class VotingBinaryHoleFillingImageFilter(SEMLikeCommandLine):
    """title: Voting Binary Hole Filling Image Filter

category: Filtering

description: Applies a voting operation in order to fill-in cavities. This can be used for smoothing contours and for filling holes in binary images. This technique is used frequently when segmenting complete organs that may have ducts or vasculature that may not have been included in the initial segmentation, e.g. lungs, kidneys, liver.

version: 0.1.0.$Revision: 19608 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/VotingBinaryHoleFillingImageFilter

contributor: Bill Lorensen (GE)

acknowledgements: This command module was derived from Insight/Examples/Filtering/VotingBinaryHoleFillingImageFilter (copyright) Insight Software Consortium

"""

    input_spec = VotingBinaryHoleFillingImageFilterInputSpec
    output_spec = VotingBinaryHoleFillingImageFilterOutputSpec
    _cmd = "VotingBinaryHoleFillingImageFilter "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Provide a base interface to AFNI commands."""
import os
from sys import platform
from distutils import spawn

from ... import logging, LooseVersion
from ...utils.filemanip import split_filename, fname_presuffix
from ..base import (
    CommandLine,
    traits,
    CommandLineInputSpec,
    isdefined,
    File,
    TraitedSpec,
    PackageInfo,
)
from ...external.due import BibTeX

# Use nipype's logging system
IFLOGGER = logging.getLogger("nipype.interface")


class Info(PackageInfo):
    """Handle afni output type and version information."""

    __outputtype = "AFNI"
    ftypes = {"NIFTI": ".nii", "AFNI": "", "NIFTI_GZ": ".nii.gz"}
    version_cmd = "afni --version"

    @staticmethod
    def parse_version(raw_info):
        """Check and parse AFNI's version."""
        version_stamp = raw_info.split("\n")[0].split("Version ")[1]
        if version_stamp.startswith("AFNI"):
            version_stamp = version_stamp.split("AFNI_")[1]
        elif version_stamp.startswith("Debian"):
            version_stamp = version_stamp.split("Debian-")[1].split("~")[0]
        else:
            return None

        version = LooseVersion(version_stamp.replace("_", ".")).version[:3]
        if version[0] < 1000:
            version[0] = version[0] + 2000
        return tuple(version)

    @classmethod
    def output_type_to_ext(cls, outputtype):
        """
        Get the file extension for the given output type.

        Parameters
        ----------
        outputtype : {'NIFTI', 'NIFTI_GZ', 'AFNI'}
            String specifying the output type.

        Returns
        -------
        extension : str
            The file extension for the output type.

        """
        try:
            return cls.ftypes[outputtype]
        except KeyError as e:
            msg = "Invalid AFNIOUTPUTTYPE: ", outputtype
            raise KeyError(msg) from e

    @classmethod
    def outputtype(cls):
        """
        Set default output filetype.

        AFNI has no environment variables, Output filetypes get set in command line calls
        Nipype uses ``AFNI`` as default


        Returns
        -------
        None

        """
        return "AFNI"

    @staticmethod
    def standard_image(img_name):
        """
        Grab an image from the standard location.

        Could be made more fancy to allow for more relocatability

        """
        clout = CommandLine(
            "which afni",
            ignore_exception=True,
            resource_monitor=False,
            terminal_output="allatonce",
        ).run()
        if clout.runtime.returncode is not 0:
            return None

        out = clout.runtime.stdout
        basedir = os.path.split(out)[0]
        return os.path.join(basedir, img_name)


class AFNICommandBase(CommandLine):
    """
    A base class to fix a linking problem in OSX and AFNI.

    See Also
    --------
    `This thread
    <http://afni.nimh.nih.gov/afni/community/board/read.php?1,145346,145347#msg-145347>`__
    about the particular environment variable that fixes this problem.

    """

    def _run_interface(self, runtime, correct_return_codes=(0,)):
        if platform == "darwin":
            runtime.environ["DYLD_FALLBACK_LIBRARY_PATH"] = "/usr/local/afni/"
        return super(AFNICommandBase, self)._run_interface(
            runtime, correct_return_codes
        )


class AFNICommandInputSpec(CommandLineInputSpec):
    num_threads = traits.Int(
        1, usedefault=True, nohash=True, desc="set number of threads"
    )
    outputtype = traits.Enum(
        "AFNI", list(Info.ftypes.keys()), desc="AFNI output filetype"
    )
    out_file = File(
        name_template="%s_afni",
        desc="output image file name",
        argstr="-prefix %s",
        name_source=["in_file"],
    )


class AFNICommandOutputSpec(TraitedSpec):
    out_file = File(desc="output file", exists=True)


class AFNICommand(AFNICommandBase):
    """Shared options for several AFNI commands."""

    input_spec = AFNICommandInputSpec
    _outputtype = None

    references_ = [
        {
            "entry": BibTeX(
                "@article{Cox1996,"
                "author={R.W. Cox},"
                "title={AFNI: software for analysis and "
                "visualization of functional magnetic "
                "resonance neuroimages},"
                "journal={Computers and Biomedical research},"
                "volume={29},"
                "number={3},"
                "pages={162-173},"
                "year={1996},"
                "}"
            ),
            "tags": ["implementation"],
        },
        {
            "entry": BibTeX(
                "@article{CoxHyde1997,"
                "author={R.W. Cox and J.S. Hyde},"
                "title={Software tools for analysis and "
                "visualization of fMRI data},"
                "journal={NMR in Biomedicine},"
                "volume={10},"
                "number={45},"
                "pages={171-178},"
                "year={1997},"
                "}"
            ),
            "tags": ["implementation"],
        },
    ]

    @property
    def num_threads(self):
        """Get number of threads."""
        return self.inputs.num_threads

    @num_threads.setter
    def num_threads(self, value):
        self.inputs.num_threads = value

    @classmethod
    def set_default_output_type(cls, outputtype):
        """
        Set the default output type for AFNI classes.

        This method is used to set the default output type for all afni
        subclasses.  However, setting this will not update the output
        type for any existing instances.  For these, assign the
        <instance>.inputs.outputtype.
        """
        if outputtype in Info.ftypes:
            cls._outputtype = outputtype
        else:
            raise AttributeError("Invalid AFNI outputtype: %s" % outputtype)

    def __init__(self, **inputs):
        """Instantiate an AFNI command tool wrapper."""
        super(AFNICommand, self).__init__(**inputs)
        self.inputs.on_trait_change(self._output_update, "outputtype")

        if hasattr(self.inputs, "num_threads"):
            self.inputs.on_trait_change(self._nthreads_update, "num_threads")

        if self._outputtype is None:
            self._outputtype = Info.outputtype()

        if not isdefined(self.inputs.outputtype):
            self.inputs.outputtype = self._outputtype
        else:
            self._output_update()

    def _nthreads_update(self):
        """Update environment with new number of threads."""
        self.inputs.environ["OMP_NUM_THREADS"] = "%d" % self.inputs.num_threads

    def _output_update(self):
        """
        Update the internal property with the provided input.

        i think? updates class private attribute based on instance input
        in fsl also updates ENVIRON variable....not valid in afni
        as it uses no environment variables
        """
        self._outputtype = self.inputs.outputtype

    def _overload_extension(self, value, name=None):
        path, base, _ = split_filename(value)
        return os.path.join(
            path, base + Info.output_type_to_ext(self.inputs.outputtype)
        )

    def _list_outputs(self):
        outputs = super(AFNICommand, self)._list_outputs()
        metadata = dict(name_source=lambda t: t is not None)
        out_names = list(self.inputs.traits(**metadata).keys())
        if out_names:
            for name in out_names:
                if outputs[name]:
                    _, _, ext = split_filename(outputs[name])
                    if ext == "":
                        outputs[name] = outputs[name] + "+orig.BRIK"
        return outputs

    def _gen_fname(self, basename, cwd=None, suffix=None, change_ext=True, ext=None):
        """
        Generate a filename based on the given parameters.

        The filename will take the form: cwd/basename<suffix><ext>.
        If change_ext is True, it will use the extentions specified in
        <instance>intputs.output_type.

        Parameters
        ----------
        basename : str
            Filename to base the new filename on.
        cwd : str
            Path to prefix to the new filename. (default is os.getcwd())
        suffix : str
            Suffix to add to the `basename`.  (defaults is '' )
        change_ext : bool
            Flag to change the filename extension to the FSL output type.
            (default True)

        Returns
        -------
        fname : str
            New filename based on given parameters.

        """
        if not basename:
            msg = "Unable to generate filename for command %s. " % self.cmd
            msg += "basename is not set!"
            raise ValueError(msg)

        if cwd is None:
            cwd = os.getcwd()
        if ext is None:
            ext = Info.output_type_to_ext(self.inputs.outputtype)
        if change_ext:
            suffix = "".join((suffix, ext)) if suffix else ext

        if suffix is None:
            suffix = ""
        fname = fname_presuffix(basename, suffix=suffix, use_ext=False, newpath=cwd)
        return fname


class AFNIPythonCommandInputSpec(CommandLineInputSpec):
    outputtype = traits.Enum(
        "AFNI", list(Info.ftypes.keys()), desc="AFNI output filetype"
    )
    py27_path = traits.Either(
        "python2", File(exists=True), usedefault=True, default="python2"
    )


class AFNIPythonCommand(AFNICommand):
    """A subtype of AFNI command line for Python scripts."""

    @property
    def cmd(self):
        """Revise the command path."""
        orig_cmd = super(AFNIPythonCommand, self).cmd
        found = spawn.find_executable(orig_cmd)
        return found if found is not None else orig_cmd

    @property
    def _cmd_prefix(self):
        return "{} ".format(self.inputs.py27_path)


def no_afni():
    """Check whether AFNI is not available."""
    if Info.version() is None:
        return True
    return False



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..segmentation import DenoiseImage


def test_DenoiseImage_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        dimension=dict(
            argstr="-d %d",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        input_image=dict(
            argstr="-i %s",
            extensions=None,
            mandatory=True,
        ),
        noise_image=dict(
            extensions=None,
            hash_files=False,
            keep_extension=True,
            name_source=["input_image"],
            name_template="%s_noise",
        ),
        noise_model=dict(
            argstr="-n %s",
            usedefault=True,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        output_image=dict(
            argstr="-o %s",
            extensions=None,
            hash_files=False,
            keep_extension=True,
            name_source=["input_image"],
            name_template="%s_noise_corrected",
        ),
        save_noise=dict(
            mandatory=True,
            usedefault=True,
            xor=["noise_image"],
        ),
        shrink_factor=dict(
            argstr="-s %s",
            usedefault=True,
        ),
        verbose=dict(
            argstr="-v",
        ),
    )
    inputs = DenoiseImage.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_DenoiseImage_outputs():
    output_map = dict(
        noise_image=dict(
            extensions=None,
        ),
        output_image=dict(
            extensions=None,
        ),
    )
    outputs = DenoiseImage.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
from .specialized import (
    ACPCTransform,
    FiducialRegistration,
    VBRAINSDemonWarp,
    BRAINSDemonWarp,
)
from .brainsresample import BRAINSResample
from .brainsfit import BRAINSFit



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)
import os


class CheckerBoardFilterInputSpec(CommandLineInputSpec):
    checkerPattern = InputMultiPath(
        traits.Int,
        desc="The pattern of input 1 and input 2 in the output image. The user can specify the number of checkers in each dimension. A checkerPattern of 2,2,1 means that images will alternate in every other checker in the first two dimensions. The same pattern will be used in the 3rd dimension.",
        sep=",",
        argstr="--checkerPattern %s",
    )
    inputVolume1 = File(
        position=-3, desc="First Input volume", exists=True, argstr="%s"
    )
    inputVolume2 = File(
        position=-2, desc="Second Input volume", exists=True, argstr="%s"
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Output filtered",
        argstr="%s",
    )


class CheckerBoardFilterOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Output filtered", exists=True)


class CheckerBoardFilter(SEMLikeCommandLine):
    """title: CheckerBoard Filter

category: Filtering

description: Create a checkerboard volume of two volumes. The output volume will show the two inputs alternating according to the user supplied checkerPattern. This filter is often used to compare the results of image registration. Note that the second input is resampled to the same origin, spacing and direction before it is composed with the first input. The scalar type of the output volume will be the same as the input image scalar type.

version: 0.1.0.$Revision: 19608 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/CheckerBoard

contributor: Bill Lorensen (GE)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = CheckerBoardFilterInputSpec
    output_spec = CheckerBoardFilterOutputSpec
    _cmd = "CheckerBoardFilter "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..minc import Dump


def test_Dump_inputs():
    input_map = dict(
        annotations_brief=dict(
            argstr="-b %s",
            xor=("annotations_brief", "annotations_full"),
        ),
        annotations_full=dict(
            argstr="-f %s",
            xor=("annotations_brief", "annotations_full"),
        ),
        args=dict(
            argstr="%s",
        ),
        coordinate_data=dict(
            argstr="-c",
            xor=("coordinate_data", "header_data"),
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        header_data=dict(
            argstr="-h",
            xor=("coordinate_data", "header_data"),
        ),
        input_file=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        line_length=dict(
            argstr="-l %d",
        ),
        netcdf_name=dict(
            argstr="-n %s",
        ),
        out_file=dict(
            argstr="> %s",
            extensions=None,
            genfile=True,
            position=-1,
        ),
        output_file=dict(
            extensions=None,
            hash_files=False,
            keep_extension=False,
            name_source=["input_file"],
            name_template="%s_dump.txt",
            position=-1,
        ),
        precision=dict(
            argstr="%s",
        ),
        variables=dict(
            argstr="-v %s",
            sep=",",
        ),
    )
    inputs = Dump.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Dump_outputs():
    output_map = dict(
        output_file=dict(
            extensions=None,
        ),
    )
    outputs = Dump.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)
import os


class BSplineDeformableRegistrationInputSpec(CommandLineInputSpec):
    iterations = traits.Int(desc="Number of iterations", argstr="--iterations %d")
    gridSize = traits.Int(
        desc="Number of grid points on interior of the fixed image. Larger grid sizes allow for finer registrations.",
        argstr="--gridSize %d",
    )
    histogrambins = traits.Int(
        desc="Number of histogram bins to use for Mattes Mutual Information. Reduce the number of bins if a deformable registration fails. If the number of bins is too large, the estimated PDFs will be a field of impulses and will inhibit reliable registration estimation.",
        argstr="--histogrambins %d",
    )
    spatialsamples = traits.Int(
        desc="Number of spatial samples to use in estimating Mattes Mutual Information. Larger values yield more accurate PDFs and improved registration quality.",
        argstr="--spatialsamples %d",
    )
    constrain = traits.Bool(
        desc="Constrain the deformation to the amount specified in Maximum Deformation",
        argstr="--constrain ",
    )
    maximumDeformation = traits.Float(
        desc="If Constrain Deformation is checked, limit the deformation to this amount.",
        argstr="--maximumDeformation %f",
    )
    default = traits.Int(
        desc="Default pixel value used if resampling a pixel outside of the volume.",
        argstr="--default %d",
    )
    initialtransform = File(
        desc="Initial transform for aligning the fixed and moving image. Maps positions in the fixed coordinate frame to positions in the moving coordinate frame. This transform should be an affine or rigid transform.  It is used an a bulk transform for the BSpline. Optional.",
        exists=True,
        argstr="--initialtransform %s",
    )
    FixedImageFileName = File(
        position=-2, desc="Fixed image to which to register", exists=True, argstr="%s"
    )
    MovingImageFileName = File(
        position=-1, desc="Moving image", exists=True, argstr="%s"
    )
    outputtransform = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Transform calculated that aligns the fixed and moving image. Maps positions from the fixed coordinate frame to the moving coordinate frame. Optional (specify an output transform or an output volume or both).",
        argstr="--outputtransform %s",
    )
    outputwarp = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Vector field that applies an equivalent warp as the BSpline. Maps positions from the fixed coordinate frame to the moving coordinate frame. Optional.",
        argstr="--outputwarp %s",
    )
    resampledmovingfilename = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Resampled moving image to fixed image coordinate frame. Optional (specify an output transform or an output volume or both).",
        argstr="--resampledmovingfilename %s",
    )


class BSplineDeformableRegistrationOutputSpec(TraitedSpec):
    outputtransform = File(
        desc="Transform calculated that aligns the fixed and moving image. Maps positions from the fixed coordinate frame to the moving coordinate frame. Optional (specify an output transform or an output volume or both).",
        exists=True,
    )
    outputwarp = File(
        desc="Vector field that applies an equivalent warp as the BSpline. Maps positions from the fixed coordinate frame to the moving coordinate frame. Optional.",
        exists=True,
    )
    resampledmovingfilename = File(
        desc="Resampled moving image to fixed image coordinate frame. Optional (specify an output transform or an output volume or both).",
        exists=True,
    )


class BSplineDeformableRegistration(SEMLikeCommandLine):
    """title: BSpline Deformable Registration

category: Legacy.Registration

description: Registers two images together using BSpline transform and mutual information.

version: 0.1.0.$Revision: 19608 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/BSplineDeformableRegistration

contributor: Bill Lorensen (GE)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = BSplineDeformableRegistrationInputSpec
    output_spec = BSplineDeformableRegistrationOutputSpec
    _cmd = "BSplineDeformableRegistration "
    _outputs_filenames = {
        "resampledmovingfilename": "resampledmovingfilename.nii",
        "outputtransform": "outputtransform.txt",
        "outputwarp": "outputwarp.nrrd",
    }


class AffineRegistrationInputSpec(CommandLineInputSpec):
    fixedsmoothingfactor = traits.Int(
        desc="Amount of smoothing applied to fixed image prior to registration. Default is 0 (none). Range is 0-5 (unitless). Consider smoothing the input data if there is considerable amounts of noise or the noise pattern in the fixed and moving images is very different.",
        argstr="--fixedsmoothingfactor %d",
    )
    movingsmoothingfactor = traits.Int(
        desc="Amount of smoothing applied to moving image prior to registration. Default is 0 (none). Range is 0-5 (unitless). Consider smoothing the input data if there is considerable amounts of noise or the noise pattern in the fixed and moving images is very different.",
        argstr="--movingsmoothingfactor %d",
    )
    histogrambins = traits.Int(
        desc="Number of histogram bins to use for Mattes Mutual Information. Reduce the number of bins if a registration fails. If the number of bins is too large, the estimated PDFs will be a field of impulses and will inhibit reliable registration estimation.",
        argstr="--histogrambins %d",
    )
    spatialsamples = traits.Int(
        desc="Number of spatial samples to use in estimating Mattes Mutual Information. Larger values yield more accurate PDFs and improved registration quality.",
        argstr="--spatialsamples %d",
    )
    iterations = traits.Int(desc="Number of iterations", argstr="--iterations %d")
    translationscale = traits.Float(
        desc="Relative scale of translations to rotations, i.e. a value of 100 means 10mm = 1 degree. (Actual scale used is 1/(TranslationScale^2)). This parameter is used to 'weight' or 'standardized' the transform parameters and their effect on the registration objective function.",
        argstr="--translationscale %f",
    )
    initialtransform = File(
        desc="Initial transform for aligning the fixed and moving image.  Maps positions in the fixed coordinate frame to positions in the moving coordinate frame. Optional.",
        exists=True,
        argstr="--initialtransform %s",
    )
    FixedImageFileName = File(
        position=-2, desc="Fixed image to which to register", exists=True, argstr="%s"
    )
    MovingImageFileName = File(
        position=-1, desc="Moving image", exists=True, argstr="%s"
    )
    outputtransform = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Transform calculated that aligns the fixed and moving image. Maps positions in the fixed coordinate frame to the moving coordinate frame. Optional (specify an output transform or an output volume or both).",
        argstr="--outputtransform %s",
    )
    resampledmovingfilename = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Resampled moving image to the fixed image coordinate frame. Optional (specify an output transform or an output volume or both).",
        argstr="--resampledmovingfilename %s",
    )


class AffineRegistrationOutputSpec(TraitedSpec):
    outputtransform = File(
        desc="Transform calculated that aligns the fixed and moving image. Maps positions in the fixed coordinate frame to the moving coordinate frame. Optional (specify an output transform or an output volume or both).",
        exists=True,
    )
    resampledmovingfilename = File(
        desc="Resampled moving image to the fixed image coordinate frame. Optional (specify an output transform or an output volume or both).",
        exists=True,
    )


class AffineRegistration(SEMLikeCommandLine):
    """title: Affine Registration

category: Legacy.Registration

description: Registers two images together using an affine transform and mutual information. This module is often used to align images of different subjects or images of the same subject from different modalities.

This module can smooth images prior to registration to mitigate noise and improve convergence. Many of the registration parameters require a working knowledge of the algorithm although the default parameters are sufficient for many registration tasks.



version: 0.1.0.$Revision: 19608 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/AffineRegistration

contributor: Daniel Blezek (GE)

acknowledgements: This module was developed by Daniel Blezek while at GE Research with contributions from Jim Miller.

This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = AffineRegistrationInputSpec
    output_spec = AffineRegistrationOutputSpec
    _cmd = "AffineRegistration "
    _outputs_filenames = {
        "resampledmovingfilename": "resampledmovingfilename.nii",
        "outputtransform": "outputtransform.txt",
    }


class MultiResolutionAffineRegistrationInputSpec(CommandLineInputSpec):
    fixedImage = File(
        position=-2,
        desc="Image which defines the space into which the moving image is registered",
        exists=True,
        argstr="%s",
    )
    movingImage = File(
        position=-1,
        desc="The transform goes from the fixed image's space into the moving image's space",
        exists=True,
        argstr="%s",
    )
    resampledImage = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Registration results",
        argstr="--resampledImage %s",
    )
    saveTransform = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Save the output transform from the registration",
        argstr="--saveTransform %s",
    )
    fixedImageMask = File(
        desc="Label image which defines a mask of interest for the fixed image",
        exists=True,
        argstr="--fixedImageMask %s",
    )
    fixedImageROI = traits.List(
        desc="Label image which defines a ROI of interest for the fixed image",
        argstr="--fixedImageROI %s",
    )
    numIterations = traits.Int(
        desc="Number of iterations to run at each resolution level.",
        argstr="--numIterations %d",
    )
    numLineIterations = traits.Int(
        desc="Number of iterations to run at each resolution level.",
        argstr="--numLineIterations %d",
    )
    stepSize = traits.Float(
        desc="The maximum step size of the optimizer in voxels", argstr="--stepSize %f"
    )
    stepTolerance = traits.Float(
        desc="The maximum step size of the optimizer in voxels",
        argstr="--stepTolerance %f",
    )
    metricTolerance = traits.Float(argstr="--metricTolerance %f")


class MultiResolutionAffineRegistrationOutputSpec(TraitedSpec):
    resampledImage = File(desc="Registration results", exists=True)
    saveTransform = File(
        desc="Save the output transform from the registration", exists=True
    )


class MultiResolutionAffineRegistration(SEMLikeCommandLine):
    """title: Robust Multiresolution Affine Registration

category: Legacy.Registration

description: Provides affine registration using multiple resolution levels and decomposed affine transforms.

version: 0.1.0.$Revision: 2104 $(alpha)

documentation-url: http://www.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/MultiResolutionAffineRegistration

contributor: Casey B Goodlett (Utah)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = MultiResolutionAffineRegistrationInputSpec
    output_spec = MultiResolutionAffineRegistrationOutputSpec
    _cmd = "MultiResolutionAffineRegistration "
    _outputs_filenames = {
        "resampledImage": "resampledImage.nii",
        "saveTransform": "saveTransform.txt",
    }


class RigidRegistrationInputSpec(CommandLineInputSpec):
    fixedsmoothingfactor = traits.Int(
        desc="Amount of smoothing applied to fixed image prior to registration. Default is 0 (none). Range is 0-5 (unitless). Consider smoothing the input data if there is considerable amounts of noise or the noise pattern in the fixed and moving images is very different.",
        argstr="--fixedsmoothingfactor %d",
    )
    movingsmoothingfactor = traits.Int(
        desc="Amount of smoothing applied to moving image prior to registration. Default is 0 (none). Range is 0-5 (unitless). Consider smoothing the input data if there is considerable amounts of noise or the noise pattern in the fixed and moving images is very different.",
        argstr="--movingsmoothingfactor %d",
    )
    testingmode = traits.Bool(
        desc="Enable testing mode. Input transform will be used to construct floating image. The floating image will be ignored if passed.",
        argstr="--testingmode ",
    )
    histogrambins = traits.Int(
        desc="Number of histogram bins to use for Mattes Mutual Information. Reduce the number of bins if a registration fails. If the number of bins is too large, the estimated PDFs will be a field of impulses and will inhibit reliable registration estimation.",
        argstr="--histogrambins %d",
    )
    spatialsamples = traits.Int(
        desc="Number of spatial samples to use in estimating Mattes Mutual Information. Larger values yield more accurate PDFs and improved registration quality.",
        argstr="--spatialsamples %d",
    )
    iterations = InputMultiPath(
        traits.Int,
        desc="Comma separated list of iterations. Must have the same number of elements as the learning rate.",
        sep=",",
        argstr="--iterations %s",
    )
    learningrate = InputMultiPath(
        traits.Float,
        desc="Comma separated list of learning rates. Learning rate is a scale factor on the gradient of the registration objective function (gradient with respect to the parameters of the transformation) used to update the parameters of the transformation during optimization. Smaller values cause the optimizer to take smaller steps through the parameter space. Larger values are typically used early in the registration process to take large jumps in parameter space followed by smaller values to home in on the optimum value of the registration objective function. Default is: 0.01, 0.005, 0.0005, 0.0002. Must have the same number of elements as iterations.",
        sep=",",
        argstr="--learningrate %s",
    )
    translationscale = traits.Float(
        desc="Relative scale of translations to rotations, i.e. a value of 100 means 10mm = 1 degree. (Actual scale used 1/(TranslationScale^2)). This parameter is used to 'weight' or 'standardized' the transform parameters and their effect on the registration objective function.",
        argstr="--translationscale %f",
    )
    initialtransform = File(
        desc="Initial transform for aligning the fixed and moving image. Maps positions in the fixed coordinate frame to positions in the moving coordinate frame. Optional.",
        exists=True,
        argstr="--initialtransform %s",
    )
    FixedImageFileName = File(
        position=-2, desc="Fixed image to which to register", exists=True, argstr="%s"
    )
    MovingImageFileName = File(
        position=-1, desc="Moving image", exists=True, argstr="%s"
    )
    outputtransform = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Transform calculated that aligns the fixed and moving image. Maps positions in the fixed coordinate frame to the moving coordinate frame. Optional (specify an output transform or an output volume or both).",
        argstr="--outputtransform %s",
    )
    resampledmovingfilename = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Resampled moving image to the fixed image coordinate frame. Optional (specify an output transform or an output volume or both).",
        argstr="--resampledmovingfilename %s",
    )


class RigidRegistrationOutputSpec(TraitedSpec):
    outputtransform = File(
        desc="Transform calculated that aligns the fixed and moving image. Maps positions in the fixed coordinate frame to the moving coordinate frame. Optional (specify an output transform or an output volume or both).",
        exists=True,
    )
    resampledmovingfilename = File(
        desc="Resampled moving image to the fixed image coordinate frame. Optional (specify an output transform or an output volume or both).",
        exists=True,
    )


class RigidRegistration(SEMLikeCommandLine):
    """title: Rigid Registration

category: Legacy.Registration

description: Registers two images together using a rigid transform and mutual information.

This module was originally distributed as "Linear registration" but has been renamed to eliminate confusion with the "Affine registration" module.

This module is often used to align images of different subjects or images of the same subject from different modalities.

This module can smooth images prior to registration to mitigate noise and improve convergence. Many of the registration parameters require a working knowledge of the algorithm although the default parameters are sufficient for many registration tasks.



version: 0.1.0.$Revision: 19608 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/RigidRegistration

contributor: Daniel Blezek (GE)

acknowledgements: This module was developed by Daniel Blezek while at GE Research with contributions from Jim Miller.

This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = RigidRegistrationInputSpec
    output_spec = RigidRegistrationOutputSpec
    _cmd = "RigidRegistration "
    _outputs_filenames = {
        "resampledmovingfilename": "resampledmovingfilename.nii",
        "outputtransform": "outputtransform.txt",
    }


class LinearRegistrationInputSpec(CommandLineInputSpec):
    fixedsmoothingfactor = traits.Int(
        desc="Amount of smoothing applied to fixed image prior to registration. Default is 0 (none). Range is 0-5 (unitless). Consider smoothing the input data if there is considerable amounts of noise or the noise pattern in the fixed and moving images is very different.",
        argstr="--fixedsmoothingfactor %d",
    )
    movingsmoothingfactor = traits.Int(
        desc="Amount of smoothing applied to moving image prior to registration. Default is 0 (none). Range is 0-5 (unitless). Consider smoothing the input data if there is considerable amounts of noise or the noise pattern in the fixed and moving images is very different.",
        argstr="--movingsmoothingfactor %d",
    )
    histogrambins = traits.Int(
        desc="Number of histogram bins to use for Mattes Mutual Information. Reduce the number of bins if a registration fails. If the number of bins is too large, the estimated PDFs will be a field of impulses and will inhibit reliable registration estimation.",
        argstr="--histogrambins %d",
    )
    spatialsamples = traits.Int(
        desc="Number of spatial samples to use in estimating Mattes Mutual Information. Larger values yield more accurate PDFs and improved registration quality.",
        argstr="--spatialsamples %d",
    )
    iterations = InputMultiPath(
        traits.Int,
        desc="Comma separated list of iterations. Must have the same number of elements as the learning rate.",
        sep=",",
        argstr="--iterations %s",
    )
    learningrate = InputMultiPath(
        traits.Float,
        desc="Comma separated list of learning rates. Learning rate is a scale factor on the gradient of the registration objective function (gradient with respect to the parameters of the transformation) used to update the parameters of the transformation during optimization. Smaller values cause the optimizer to take smaller steps through the parameter space. Larger values are typically used early in the registration process to take large jumps in parameter space followed by smaller values to home in on the optimum value of the registration objective function. Default is: 0.01, 0.005, 0.0005, 0.0002. Must have the same number of elements as iterations.",
        sep=",",
        argstr="--learningrate %s",
    )
    translationscale = traits.Float(
        desc="Relative scale of translations to rotations, i.e. a value of 100 means 10mm = 1 degree. (Actual scale used 1/(TranslationScale^2)). This parameter is used to 'weight' or 'standardized' the transform parameters and their effect on the registration objective function.",
        argstr="--translationscale %f",
    )
    initialtransform = File(
        desc="Initial transform for aligning the fixed and moving image. Maps positions in the fixed coordinate frame to positions in the moving coordinate frame. Optional.",
        exists=True,
        argstr="--initialtransform %s",
    )
    FixedImageFileName = File(
        position=-2, desc="Fixed image to which to register", exists=True, argstr="%s"
    )
    MovingImageFileName = File(
        position=-1, desc="Moving image", exists=True, argstr="%s"
    )
    outputtransform = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Transform calculated that aligns the fixed and moving image. Maps positions in the fixed coordinate frame to the moving coordinate frame. Optional (specify an output transform or an output volume or both).",
        argstr="--outputtransform %s",
    )
    resampledmovingfilename = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Resampled moving image to the fixed image coordinate frame. Optional (specify an output transform or an output volume or both).",
        argstr="--resampledmovingfilename %s",
    )


class LinearRegistrationOutputSpec(TraitedSpec):
    outputtransform = File(
        desc="Transform calculated that aligns the fixed and moving image. Maps positions in the fixed coordinate frame to the moving coordinate frame. Optional (specify an output transform or an output volume or both).",
        exists=True,
    )
    resampledmovingfilename = File(
        desc="Resampled moving image to the fixed image coordinate frame. Optional (specify an output transform or an output volume or both).",
        exists=True,
    )


class LinearRegistration(SEMLikeCommandLine):
    """title: Linear Registration

category: Legacy.Registration

description: Registers two images together using a rigid transform and mutual information.

version: 0.1.0.$Revision: 19608 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/LinearRegistration

contributor: Daniel Blezek (GE)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = LinearRegistrationInputSpec
    output_spec = LinearRegistrationOutputSpec
    _cmd = "LinearRegistration "
    _outputs_filenames = {
        "resampledmovingfilename": "resampledmovingfilename.nii",
        "outputtransform": "outputtransform.txt",
    }


class ExpertAutomatedRegistrationInputSpec(CommandLineInputSpec):
    fixedImage = File(
        position=-2,
        desc="Image which defines the space into which the moving image is registered",
        exists=True,
        argstr="%s",
    )
    movingImage = File(
        position=-1,
        desc="The transform goes from the fixed image's space into the moving image's space",
        exists=True,
        argstr="%s",
    )
    resampledImage = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Registration results",
        argstr="--resampledImage %s",
    )
    loadTransform = File(
        desc="Load a transform that is immediately applied to the moving image",
        exists=True,
        argstr="--loadTransform %s",
    )
    saveTransform = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Save the transform that results from registration",
        argstr="--saveTransform %s",
    )
    initialization = traits.Enum(
        "None",
        "Landmarks",
        "ImageCenters",
        "CentersOfMass",
        "SecondMoments",
        desc="Method to prime the registration process",
        argstr="--initialization %s",
    )
    registration = traits.Enum(
        "None",
        "Initial",
        "Rigid",
        "Affine",
        "BSpline",
        "PipelineRigid",
        "PipelineAffine",
        "PipelineBSpline",
        desc="Method for the registration process",
        argstr="--registration %s",
    )
    metric = traits.Enum(
        "MattesMI",
        "NormCorr",
        "MeanSqrd",
        desc="Method to quantify image match",
        argstr="--metric %s",
    )
    expectedOffset = traits.Float(
        desc="Expected misalignment after initialization", argstr="--expectedOffset %f"
    )
    expectedRotation = traits.Float(
        desc="Expected misalignment after initialization",
        argstr="--expectedRotation %f",
    )
    expectedScale = traits.Float(
        desc="Expected misalignment after initialization", argstr="--expectedScale %f"
    )
    expectedSkew = traits.Float(
        desc="Expected misalignment after initialization", argstr="--expectedSkew %f"
    )
    verbosityLevel = traits.Enum(
        "Silent",
        "Standard",
        "Verbose",
        desc="Level of detail of reporting progress",
        argstr="--verbosityLevel %s",
    )
    sampleFromOverlap = traits.Bool(
        desc="Limit metric evaluation to the fixed image region overlapped by the moving image",
        argstr="--sampleFromOverlap ",
    )
    fixedImageMask = File(
        desc="Image which defines a mask for the fixed image",
        exists=True,
        argstr="--fixedImageMask %s",
    )
    randomNumberSeed = traits.Int(
        desc="Seed to generate a consistent random number sequence",
        argstr="--randomNumberSeed %d",
    )
    numberOfThreads = traits.Int(
        desc="Number of CPU threads to use", argstr="--numberOfThreads %d"
    )
    minimizeMemory = traits.Bool(
        desc="Reduce the amount of memory required at the cost of increased computation time",
        argstr="--minimizeMemory ",
    )
    interpolation = traits.Enum(
        "NearestNeighbor",
        "Linear",
        "BSpline",
        desc="Method for interpolation within the optimization process",
        argstr="--interpolation %s",
    )
    fixedLandmarks = InputMultiPath(
        traits.List(traits.Float(), minlen=3, maxlen=3),
        desc="Ordered list of landmarks in the fixed image",
        argstr="--fixedLandmarks %s...",
    )
    movingLandmarks = InputMultiPath(
        traits.List(traits.Float(), minlen=3, maxlen=3),
        desc="Ordered list of landmarks in the moving image",
        argstr="--movingLandmarks %s...",
    )
    rigidMaxIterations = traits.Int(
        desc="Maximum number of rigid optimization iterations",
        argstr="--rigidMaxIterations %d",
    )
    rigidSamplingRatio = traits.Float(
        desc="Portion of the image to use in computing the metric during rigid registration",
        argstr="--rigidSamplingRatio %f",
    )
    affineMaxIterations = traits.Int(
        desc="Maximum number of affine optimization iterations",
        argstr="--affineMaxIterations %d",
    )
    affineSamplingRatio = traits.Float(
        desc="Portion of the image to use in computing the metric during affine registration",
        argstr="--affineSamplingRatio %f",
    )
    bsplineMaxIterations = traits.Int(
        desc="Maximum number of bspline optimization iterations",
        argstr="--bsplineMaxIterations %d",
    )
    bsplineSamplingRatio = traits.Float(
        desc="Portion of the image to use in computing the metric during BSpline registration",
        argstr="--bsplineSamplingRatio %f",
    )
    controlPointSpacing = traits.Int(
        desc="Number of pixels between control points",
        argstr="--controlPointSpacing %d",
    )


class ExpertAutomatedRegistrationOutputSpec(TraitedSpec):
    resampledImage = File(desc="Registration results", exists=True)
    saveTransform = File(
        desc="Save the transform that results from registration", exists=True
    )


class ExpertAutomatedRegistration(SEMLikeCommandLine):
    """title: Expert Automated Registration

category: Legacy.Registration

description: Provides rigid, affine, and BSpline registration methods via a simple GUI

version: 0.1.0.$Revision: 2104 $(alpha)

documentation-url: http://www.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/ExpertAutomatedRegistration

contributor: Stephen R Aylward (Kitware), Casey B Goodlett (Kitware)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = ExpertAutomatedRegistrationInputSpec
    output_spec = ExpertAutomatedRegistrationOutputSpec
    _cmd = "ExpertAutomatedRegistration "
    _outputs_filenames = {
        "resampledImage": "resampledImage.nii",
        "saveTransform": "saveTransform.txt",
    }



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import os
import pytest

from nipype.interfaces import utility
import nipype.pipeline.engine as pe

concat_sort = """\
def concat_sort(in_arrays):
    import numpy as np
    all_vals = np.concatenate([arr.flatten() for arr in in_arrays])
    return np.sort(all_vals)
"""


def test_function(tmpdir):
    tmpdir.chdir()

    def gen_random_array(size):
        import numpy as np

        return np.random.rand(size, size)

    f1 = pe.MapNode(
        utility.Function(
            input_names=["size"],
            output_names=["random_array"],
            function=gen_random_array,
        ),
        name="random_array",
        iterfield=["size"],
    )
    f1.inputs.size = [2, 3, 5]

    wf = pe.Workflow(name="test_workflow")

    def increment_array(in_array):
        return in_array + 1

    f2 = pe.MapNode(
        utility.Function(function=increment_array),
        name="increment_array",
        iterfield=["in_array"],
    )

    wf.connect(f1, "random_array", f2, "in_array")

    f3 = pe.Node(utility.Function(function=concat_sort), name="concat_sort")

    wf.connect(f2, "out", f3, "in_arrays")
    wf.run()


def make_random_array(size):
    return np.random.randn(size, size)  # noqa


def should_fail(tmp):
    tmp.chdir()

    node = pe.Node(
        utility.Function(
            input_names=["size"],
            output_names=["random_array"],
            function=make_random_array,
        ),
        name="should_fail",
    )
    node.inputs.size = 10
    node.run()


def test_should_fail(tmpdir):
    with pytest.raises(NameError):
        should_fail(tmpdir)


def test_function_with_imports(tmpdir):
    tmpdir.chdir()

    node = pe.Node(
        utility.Function(
            input_names=["size"],
            output_names=["random_array"],
            function=make_random_array,
            imports=["import numpy as np"],
        ),
        name="should_not_fail",
    )
    print(node.inputs.function_str)
    node.inputs.size = 10
    node.run()


def test_aux_connect_function(tmpdir):
    """ This tests excution nodes with multiple inputs and auxiliary
    function inside the Workflow connect function.
    """
    tmpdir.chdir()

    wf = pe.Workflow(name="test_workflow")

    def _gen_tuple(size):
        return [1,] * size

    def _sum_and_sub_mul(a, b, c):
        return (a + b) * c, (a - b) * c

    def _inc(x):
        return x + 1

    params = pe.Node(utility.IdentityInterface(fields=["size", "num"]), name="params")
    params.inputs.num = 42
    params.inputs.size = 1

    gen_tuple = pe.Node(
        utility.Function(
            input_names=["size"], output_names=["tuple"], function=_gen_tuple
        ),
        name="gen_tuple",
    )

    ssm = pe.Node(
        utility.Function(
            input_names=["a", "b", "c"],
            output_names=["sum", "sub"],
            function=_sum_and_sub_mul,
        ),
        name="sum_and_sub_mul",
    )

    split = pe.Node(utility.Split(splits=[1, 1], squeeze=True), name="split")

    wf.connect(
        [
            (params, gen_tuple, [(("size", _inc), "size")]),
            (params, ssm, [(("num", _inc), "c")]),
            (gen_tuple, split, [("tuple", "inlist")]),
            (split, ssm, [(("out1", _inc), "a"), ("out2", "b"),]),
        ]
    )

    wf.run()



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""The fix module provides classes for interfacing with the `FSL FIX
<http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/FIX/index.html>` command line tools.

This was written to work with FSL version v5.0

The following example assumes that melodic has already been run, so
the datagrabber is configured to start from there (a list of melodic
output directories). If no hand_labels_noise.txt exists already, this
will fail and comment on that.

EXAMPLE:
subject_list = ['1', '2', '3']

fix_pipeline = pe.Workflow(name='fix_pipeline')
fix_pipeline.base_dir = os.path.abspath('./')

info = dict(mel_ica=[['subject_id']])

datasource = pe.Node(interface=nio.DataGrabber(infields=['subject_id'], outfields=['mel_ica']), name='datasource')
datasource.inputs.base_directory = os.path.abspath('<path_to_base_directory>')
datasource.inputs.template = '%s/<path_to_melodic_ica>'
datasource.inputs.template_args = info
datasource.inputs.subject_id =  subject_list
datasource.inputs.sort_filelist = True
datasource.iterables = ('subject_id', subject_list)

# create training set by looking into which mel_icas have hand_labels_noise.txt files in them
create_training_set = pe.JoinNode(interface=fix.TrainingSetCreator(), joinfield=['mel_icas_in'], joinsource='datasource', name='trainingset_creator')

# train the classifier
train_node = pe.Node(interface=fix.Training(trained_wts_filestem='foo'), name='train_node')

# test accuracy. Probably not necessary, and also failing on my setup because of fix itself (no error msg)
accuracy_tester = pe.Node(interface=fix.AccuracyTester(output_directory='more_foo'), name='accuracy_tester')

# classify components
classify_node = pe.Node(interface=fix.Classifier(), name='classify')

# apply cleanup
cleaner_node = pe.Node(interface=fix.Cleaner(), name='cleaner')

fix_pipeline.connect(datasource, 'mel_ica', create_training_set, 'mel_icas_in')
fix_pipeline.connect(create_training_set, 'mel_icas_out', train_node, 'mel_icas')
fix_pipeline.connect(train_node, 'trained_wts_file', accuracy_tester, 'trained_wts_file')
fix_pipeline.connect(datasource, 'mel_ica', accuracy_tester, 'mel_icas')
fix_pipeline.connect(train_node, 'trained_wts_file', classify_node, 'trained_wts_file')
fix_pipeline.connect(datasource, 'mel_ica', classify_node, 'mel_ica')
fix_pipeline.connect(classify_node, 'artifacts_list_file', cleaner_node, 'artifacts_list_file')

fix_pipeline.write_graph()
outgraph = fix_pipeline.run()

"""

from ..base import (
    TraitedSpec,
    CommandLineInputSpec,
    CommandLine,
    InputMultiPath,
    OutputMultiPath,
    BaseInterface,
    BaseInterfaceInputSpec,
    traits,
    Directory,
    File,
    isdefined,
)
import os


class TrainingSetCreatorInputSpec(BaseInterfaceInputSpec):
    mel_icas_in = InputMultiPath(
        Directory(exists=True),
        copyfile=False,
        desc="Melodic output directories",
        argstr="%s",
        position=-1,
    )


class TrainingSetCreatorOutputSpec(TraitedSpec):
    mel_icas_out = OutputMultiPath(
        Directory(exists=True),
        copyfile=False,
        desc="Hand labels for noise vs signal",
        argstr="%s",
        position=-1,
    )


class TrainingSetCreator(BaseInterface):
    """Goes through set of provided melodic output directories, to find all
    the ones that have a hand_labels_noise.txt file in them.

    This is outsourced as a separate class, so that the pipeline is
    rerun everytime a handlabeled file has been changed, or a new one
    created.

    """

    input_spec = TrainingSetCreatorInputSpec
    output_spec = TrainingSetCreatorOutputSpec
    _always_run = True

    def _run_interface(self, runtime):
        mel_icas = []
        for item in self.inputs.mel_icas_in:
            if os.path.exists(os.path.join(item, "hand_labels_noise.txt")):
                mel_icas.append(item)

        if len(mel_icas) == 0:
            raise Exception(
                "%s did not find any hand_labels_noise.txt files in the following directories: %s"
                % (self.__class__.__name__, mel_icas)
            )

        return runtime

    def _list_outputs(self):
        mel_icas = []
        for item in self.inputs.mel_icas_in:
            if os.path.exists(os.path.join(item, "hand_labels_noise.txt")):
                mel_icas.append(item)
        outputs = self._outputs().get()
        outputs["mel_icas_out"] = mel_icas
        return outputs


class FeatureExtractorInputSpec(CommandLineInputSpec):
    mel_ica = Directory(
        exists=True,
        copyfile=False,
        desc="Melodic output directory or directories",
        argstr="%s",
        position=-1,
    )


class FeatureExtractorOutputSpec(TraitedSpec):
    mel_ica = Directory(
        exists=True,
        copyfile=False,
        desc="Melodic output directory or directories",
        argstr="%s",
        position=-1,
    )


class FeatureExtractor(CommandLine):
    """
    Extract features (for later training and/or classifying)
    """

    input_spec = FeatureExtractorInputSpec
    output_spec = FeatureExtractorOutputSpec
    cmd = "fix -f"

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs["mel_ica"] = self.inputs.mel_ica
        return outputs


class TrainingInputSpec(CommandLineInputSpec):
    mel_icas = InputMultiPath(
        Directory(exists=True),
        copyfile=False,
        desc="Melodic output directories",
        argstr="%s",
        position=-1,
    )

    trained_wts_filestem = traits.Str(
        desc="trained-weights filestem, used for trained_wts_file and output directories",
        argstr="%s",
        position=1,
    )

    loo = traits.Bool(
        argstr="-l", desc="full leave-one-out test with classifier training", position=2
    )


class TrainingOutputSpec(TraitedSpec):
    trained_wts_file = File(exists=True, desc="Trained-weights file")


class Training(CommandLine):
    """
    Train the classifier based on your own FEAT/MELODIC output directory.
    """

    input_spec = TrainingInputSpec
    output_spec = TrainingOutputSpec
    cmd = "fix -t"

    def _list_outputs(self):
        outputs = self.output_spec().get()
        if isdefined(self.inputs.trained_wts_filestem):
            outputs["trained_wts_file"] = os.path.abspath(
                self.inputs.trained_wts_filestem + ".RData"
            )
        else:
            outputs["trained_wts_file"] = os.path.abspath("trained_wts_file.RData")
        return outputs


class AccuracyTesterInputSpec(CommandLineInputSpec):
    mel_icas = InputMultiPath(
        Directory(exists=True),
        copyfile=False,
        desc="Melodic output directories",
        argstr="%s",
        position=3,
        mandatory=True,
    )

    trained_wts_file = File(
        desc="trained-weights file", argstr="%s", position=1, mandatory=True
    )

    output_directory = Directory(
        desc="Path to folder in which to store the results of the accuracy test.",
        argstr="%s",
        position=2,
        mandatory=True,
    )


class AccuracyTesterOutputSpec(TraitedSpec):
    output_directory = Directory(
        desc="Path to folder in which to store the results of the accuracy test.",
        argstr="%s",
        position=1,
    )


class AccuracyTester(CommandLine):
    """
    Test the accuracy of an existing training dataset on a set of hand-labelled subjects.
    Note: This may or may not be working. Couldn't presently not confirm because fix fails on this (even outside of nipype) without leaving an error msg.
    """

    input_spec = AccuracyTesterInputSpec
    output_spec = AccuracyTesterOutputSpec
    cmd = "fix -C"

    def _list_outputs(self):
        outputs = self.output_spec().get()
        if isdefined(self.inputs.output_directory):
            outputs["output_directory"] = Directory(
                exists=False, value=self.inputs.output_directory
            )
        else:
            outputs["output_directory"] = Directory(exists=False, value="accuracy_test")
        return outputs


class ClassifierInputSpec(CommandLineInputSpec):
    mel_ica = Directory(
        exists=True,
        copyfile=False,
        desc="Melodic output directory or directories",
        argstr="%s",
        position=1,
    )

    trained_wts_file = File(
        exists=True,
        desc="trained-weights file",
        argstr="%s",
        position=2,
        mandatory=True,
        copyfile=False,
    )

    thresh = traits.Int(
        argstr="%d", desc="Threshold for cleanup.", position=-1, mandatory=True
    )

    artifacts_list_file = File(
        desc="Text file listing which ICs are artifacts; can be the output from classification or can be created manually"
    )


class ClassifierOutputSpec(TraitedSpec):
    artifacts_list_file = File(
        desc="Text file listing which ICs are artifacts; can be the output from classification or can be created manually"
    )


class Classifier(CommandLine):
    """
    Classify ICA components using a specific training dataset (<thresh> is in the range 0-100, typically 5-20).
    """

    input_spec = ClassifierInputSpec
    output_spec = ClassifierOutputSpec
    cmd = "fix -c"

    def _gen_artifacts_list_file(self, mel_ica, thresh):

        _, trained_wts_file = os.path.split(self.inputs.trained_wts_file)
        trained_wts_filestem = trained_wts_file.split(".")[0]
        filestem = "fix4melview_" + trained_wts_filestem + "_thr"

        fname = os.path.join(mel_ica, filestem + str(thresh) + ".txt")
        return fname

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs["artifacts_list_file"] = self._gen_artifacts_list_file(
            self.inputs.mel_ica, self.inputs.thresh
        )

        return outputs


class CleanerInputSpec(CommandLineInputSpec):
    artifacts_list_file = File(
        exists=True,
        argstr="%s",
        position=1,
        mandatory=True,
        desc="Text file listing which ICs are artifacts; can be the output from classification or can be created manually",
    )

    cleanup_motion = traits.Bool(
        argstr="-m",
        desc="cleanup motion confounds, looks for design.fsf for highpass filter cut-off",
        position=2,
    )

    highpass = traits.Float(
        100,
        argstr="-m -h %f",
        usedefault=True,
        desc="cleanup motion confounds",
        position=2,
    )

    aggressive = traits.Bool(
        argstr="-A",
        desc="Apply aggressive (full variance) cleanup, instead of the default less-aggressive (unique variance) cleanup.",
        position=3,
    )

    confound_file = File(
        argstr="-x %s", desc="Include additional confound file.", position=4
    )

    confound_file_1 = File(
        argstr="-x %s", desc="Include additional confound file.", position=5
    )

    confound_file_2 = File(
        argstr="-x %s", desc="Include additional confound file.", position=6
    )


class CleanerOutputSpec(TraitedSpec):
    cleaned_functional_file = File(exists=True, desc="Cleaned session data")


class Cleaner(CommandLine):
    """
    Extract features (for later training and/or classifying)
    """

    input_spec = CleanerInputSpec
    output_spec = CleanerOutputSpec
    cmd = "fix -a"

    def _get_cleaned_functional_filename(self, artifacts_list_filename):
        """ extract the proper filename from the first line of the artifacts file """
        artifacts_list_file = open(artifacts_list_filename, "r")
        functional_filename, extension = artifacts_list_file.readline().split(".")
        artifacts_list_file_path, artifacts_list_filename = os.path.split(
            artifacts_list_filename
        )

        return os.path.join(
            artifacts_list_file_path, functional_filename + "_clean.nii.gz"
        )

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs["cleaned_functional_file"] = self._get_cleaned_functional_filename(
            self.inputs.artifacts_list_file
        )
        return outputs



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)
import os


class ImageLabelCombineInputSpec(CommandLineInputSpec):
    InputLabelMap_A = File(
        position=-3, desc="Label map image", exists=True, argstr="%s"
    )
    InputLabelMap_B = File(
        position=-2, desc="Label map image", exists=True, argstr="%s"
    )
    OutputLabelMap = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Resulting Label map image",
        argstr="%s",
    )
    first_overwrites = traits.Bool(
        desc="Use first or second label when both are present",
        argstr="--first_overwrites ",
    )


class ImageLabelCombineOutputSpec(TraitedSpec):
    OutputLabelMap = File(position=-1, desc="Resulting Label map image", exists=True)


class ImageLabelCombine(SEMLikeCommandLine):
    """title: Image Label Combine

category: Filtering

description: Combine two label maps into one

version: 0.1.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/ImageLabelCombine

contributor: Alex Yarmarkovich (SPL, BWH)

"""

    input_spec = ImageLabelCombineInputSpec
    output_spec = ImageLabelCombineOutputSpec
    _cmd = "ImageLabelCombine "
    _outputs_filenames = {"OutputLabelMap": "OutputLabelMap.nii"}
